var EventEmitter = require('events').EventEmitter,
    namespaces = require('namespaces');

Profile = exports.Profile = function(username) {
    EventEmitter.call(this);

    var stores = global.m1node.stores;
    var databaseName = global.m1node.settings.profileDatabase;
    var data = stores[databaseName  || 'profile'].getDatabase();
    var profile = null;
    var self = this;
    var dirty = false;

    data.getDoc(username, function(err, rawProfile) {
        if (err) {
            rawProfile = {
                createStamp: new Date().getTime()
            };
            dirty = true;
        }
        profile = rawProfile;
        self.emit('ready');
    });

    this.__defineGetter__('username', function() {
        return username;
    });
    
    this.get = function(key) {
        if (!key) {
            throw new Error('key is required to get value from profile');
        }
        return getValue(profile, key);
    };

    this.set = function(key, val) {
        if (!key || !val) {
            throw new Error('key and value are required to set profile value');
        }
        var result = setValue(profile, key, val);
        dirty = true;
        return result;
    };
    
    this.save = function() {
        if (dirty) {
            profile.updateStamp = new Date().getTime();
            data.saveDoc(username, profile, function(err, ok) {
                dirty = false;
                self.emit('saved', err, ok);
            });
        } else {
            self.emit('saved');
        }
    };
};

Profile.prototype = Object.create(EventEmitter.prototype);
