var assert = require('assert'),
    should = require('should'),
    profile = require('m1node/profile'),
    settings = require('../example/full/settings'),
    Stores = require('m1node/storage').Stores;

var stores = new Stores(settings.databases, settings.databaseAuthFile);

var tests = {
    'test init': function(done) {
        var username = 'mikebannister1';
        var profile = new Profile(username);
        profile.addListener('ready', function() {
            profile.username.should.eql(username);
            done();
        });
    },
    
    'test get value': function(done) {
        var username = 'mikebannister2';
        var profile = new Profile(username);
        profile.addListener('ready', function() {
            profile.username.should.eql(username);
            var createStamp = profile.get('createStamp');
            createStamp.should.be.a('number');
            done();
        });
    },
    
    'test set value': function(done) {
        var username = 'mikebannister2';
        var profile = new Profile(username);
        profile.addListener('ready', function() {
            profile.set('example1.example2', 'profile string value');
            profile.get('example1').should.have.property('example2');
            profile.get('example1.example2').should.eql('profile string value');
            profile.set('example3.example4', {
                example5: 'complex profile value'
            });
            profile.get('example3').should.have.property('example4');
            profile.get('example3.example4').should.eql({
                example5: 'complex profile value'
            });
            done();
        });
    },
    
    'test dirty save': function(done) {
        var username = 'mikebannister2';
        var profile1 = new Profile(username);
        profile1.addListener('ready', function() {
            profile1.addListener('saved', function(err, ok) {
                if (err) {
                    assert.fail('could not save profile');
                } else {
                    var profile2 = new Profile(username);
                    profile2.addListener('ready', function() {
                        profile2.get('example5.example6').should.eql('profile string value');
                        var testValue = profile1.get('example5.example6');
                        testValue.should.eql('profile string value');
                        var updateStamp = profile2.get('updateStamp');
                        updateStamp.should.be.a('number');
                        done();
                    });
                }
            });
            profile1.set('example5.example6', 'profile string value');
            profile1.save();
        });
    },
    
    // NOTE: right now this depends on the previous test populating updateStamp
    'test clean save': function(done) {
        var username = 'mikebannister2';
        var profile1 = new Profile(username);
        profile1.addListener('ready', function() {
            var updateStamp = profile1.get('updateStamp');
            // wait a sec so we can compare timestamps
            setTimeout(function() {
                // no changes here so no updated should take place
                profile1.save();
            }, 1000);
            profile1.addListener('saved', function(err, ok) {
                if (err) {
                    assert.fail('could not save profile');
                } else {
                    var profile2 = new Profile(username);
                    profile2.addListener('ready', function() {
                        profile2.get('updateStamp').should.not.be.above(updateStamp);
                        done();
                    });
                }
            });
        });
    },
    
    'test misuse get': function(done) {
        var username = 'mikebannister2';
        var profile = new Profile(username);
        profile.addListener('ready', function() {
            assert.throws(function() {
                profile.get();
            }, Error);
            done();
        });
    },
    
    'test misuse set': function(done) {
        var username = 'mikebannister2';
        var profile = new Profile(username);
        profile.addListener('ready', function() {
            assert.throws(function() {
                profile.set('just a value');
            }, Error);
            done();
        });
    }
};

module.exports = tests;
