var path = require('path'),
    assert = require('assert'),
    settings = require('./files/settings'),
    should = require('should'),
    storage = require('m1node/storage'),
    couchdb = require('couchdb');

var stores,
    conf1, conf2, conf3,
    key1, key2, key3,
    client1, client2, client3,
    database1, database2, database3;

var setup = function() {
    stores = new Stores(settings.databases, settings.databaseAuthFile);

    conf1 = stores.m1nodeTest1.getConf();
    conf2 = stores.m1nodeTest2.getConf();
    conf3 = stores.m1nodeTest3.getConf();
    
    key1 = stores.m1nodeTest1.getKey();
    key2 = stores.m1nodeTest2.getKey();
    key3 = stores.m1nodeTest3.getKey();
    
    client1 = stores.m1nodeTest1.getClient();
    client2 = stores.m1nodeTest2.getClient();
    client3 = stores.m1nodeTest3.getClient();
    
    database1 = stores.m1nodeTest1.getDatabase();
    database2 = stores.m1nodeTest2.getDatabase();
    database3 = stores.m1nodeTest3.getDatabase();
};

module.exports = {
    'tests length': function(done) {
        setup();
        var keys = Object.keys(stores);
        assert.length(keys, 3, "storage should have one store for each store in config");
        done();
    },

    'tests require settings': function(done) {
        assert.throws(function() {
            stores = new Stores();
        }, Error);
        done();
    },
    
    'test key is valid': function(done) {
        assert.eql(key1, 'm1nodeTest1', "m1nodeTest1 store key should be m1nodeTest1");
        assert.eql(key2, 'm1nodeTest2', "m1nodeTest2 store key should be m1nodeTest2");
        assert.eql(key3, 'm1nodeTest3', "m1nodeTest3 store key should be m1nodeTest3");
        done();
    },
    
    'test client is object': function(done) {
        client1.should.be.an.instanceof(couchdb.Client, "m1nodeTest1 client should be a couchdb.Client instance");
        client2.should.be.an.instanceof(couchdb.Client, "m1nodeTest2 client should be a couchdb.Client instance");
        client3.should.be.an.instanceof(couchdb.Client, "m1nodeTest3 client should be a couchdb.Client instance");
        done();
    },
    
    'test database is object': function(done) {
        database1.should.be.an.instanceof(couchdb.Db, "m1nodeTest1 database should be a couchdb.Db instance");
        database2.should.be.an.instanceof(couchdb.Db, "m1nodeTest2 database should be a couchdb.Db instance");
        database3.should.be.an.instanceof(couchdb.Db, "m1nodeTest3 database should be a couchdb.Db instance");
        done();
    },
    
    'test conf host matches conf': function(done) {
        assert.eql(conf1.host, 'localhost', "m1nodeTest1 store conf host should be localhost");
        assert.eql(conf2.host, 'localhost', "m1nodeTest2 store conf host should be localhost");
        assert.eql(conf3.host, 'fakehost', "m1nodeTest3 store conf host should be fakehost");
        done();
    },
    
    'test conf port matches conf': function(done) {
        assert.eql(conf1.port,  5984, "m1nodeTest1 store conf port should be 5984");
        assert.eql(conf2.port,  5984, "m1nodeTest2 store conf port should be 5984");
        assert.eql(conf3.port,  2222, "m1nodeTest3 store conf port should be 2222");
        done();
    },
    
    'test conf name matches conf': function(done) {
        assert.eql(conf1.name, 'm1node-test1', "m1nodeTest1 store name should be m1node-test1");
        assert.eql(conf2.name, 'm1node-test2', "m1nodeTest2 store name should be m1node-test2");
        assert.eql(conf3.name, 'm1node-test3', "m1nodeTest3 store name should be m1node-test3");
        done();
    },
    
    'test username matches conf': function(done) {
        assert.eql(conf1.username, 'm1node1', "m1nodeTest1 store username should be m1node1");
        assert.eql(conf2.username, 'm1node2', "m1nodeTest2 store username should be m1node2");
        assert.eql(conf3.username, 'm1node3', "m1nodeTest3 store username should be m1node3");
        done();
    },
    
    'test password matches auth file': function(done) {
        assert.eql(conf1.password, 'm1nodeTest', "m1nodeTest1 store password should be m1nodeTest");
        assert.isUndefined(conf2.password, "m1nodeTest2 store password should be undefined");
        assert.isUndefined(conf3.password, "m1nodeTest3 store password should be undefined");
        done();
    }
};
